=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2008-08-12 19:24:49 +0000
+++ loggerhead/apps/branch.py	2008-09-09 22:47:45 +0000
@@ -25,7 +25,7 @@
 class BranchWSGIApp(object):
 
     def __init__(self, branch, friendly_name=None, config={},
-                 graph_cache=None, branch_link=None):
+                 graph_cache=None, branch_link=None, is_root=False):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -34,6 +34,7 @@
         if graph_cache is None:
             graph_cache = bzrlib.lru_cache.LRUCache()
         self.graph_cache = graph_cache
+        self.is_root = is_root
 
     def get_history(self):
         _history = History(self.branch, self.graph_cache)

=== modified file 'loggerhead/apps/filesystem.py'
--- loggerhead/apps/filesystem.py	2008-08-05 05:11:33 +0000
+++ loggerhead/apps/filesystem.py	2008-08-12 23:10:24 +0000
@@ -24,10 +24,13 @@
     def app_for_branch(self, branch):
         if not self.name:
             name = branch.nick
+            is_root = True
         else:
             name = self.name
+            is_root = False
         branch_app = BranchWSGIApp(
-            branch, name, {'cachepath': sql_dir}, self.root.graph_cache)
+            branch, name, {'cachepath': sql_dir}, self.root.graph_cache,
+            is_root=is_root)
         return branch_app.app
 
     def app_for_non_branch(self, environ):

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-07-23 21:41:49 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-09-09 22:42:27 +0000
@@ -20,7 +20,7 @@
 import os
 import posixpath
 
-from paste.httpexceptions import HTTPBadRequest
+from paste.httpexceptions import HTTPBadRequest, HTTPServerError
 
 from loggerhead.controllers import TemplatedBranchView
 from loggerhead import util
@@ -62,6 +62,21 @@
             path = h.get_path(revid, file_id)
         filename = os.path.basename(path)
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files'))
+
+        # Create breadcrumb trail for the path within the branch
+        try:
+            inv = h.get_inventory(revid)
+        except:
+            self.log.exception('Exception fetching changes')
+            raise HTTPServerError('Could not fetch changes')
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
+
         return {
             'revid': revid,
             'file_id': file_id,
@@ -71,4 +86,6 @@
             'change': h.get_changes([ revid ])[0],
             'contents': list(h.annotate_file(file_id, revid)),
             'fileview_active': True,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'branch_breadcrumbs': branch_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-08-16 21:53:52 +0000
@@ -86,6 +86,13 @@
                     all_same_author = False
                     break
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'changes': changes,
@@ -100,4 +107,5 @@
             'search_failed': search_failed,
             'all_same_author': all_same_author,
             'url': self._branch.context_url,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-08-06 21:25:24 +0000
+++ loggerhead/controllers/directory_ui.py	2008-08-16 21:53:52 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import branch
 
+from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 
 class DirEntry(object):
@@ -70,8 +71,14 @@
                 b = None
             dirs.append(DirEntry(d, parity, b))
             parity = 1 - parity
+        # Create breadcrumb trail
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._name,
+                False,
+                'directory')
         return {
             'dirs': dirs,
             'name': self._name,
             'static_url': static_url,
+            'directory_breadcrumbs': directory_breadcrumbs,
             }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-07-23 21:41:49 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-08-16 21:53:52 +0000
@@ -75,6 +75,15 @@
         if updir == '/':
             updir_file_id = None
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files')
+
+        # Create breadcrumb trail for the path within the branch
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
+        
         return {
             'branch': self._branch,
             'util': util,
@@ -91,4 +100,6 @@
             'url': self._branch.context_url,
             'start_revid': start_revid,
             'fileview_active': True,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'branch_breadcrumbs': branch_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/revision_ui.py	2008-08-16 21:53:52 +0000
@@ -72,6 +72,13 @@
         if side_by_side:
             h.add_side_by_side([ change ])
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'revid': revid,
@@ -89,4 +96,5 @@
             'line_count': line_count,
             'line_count_limit': line_count_limit,
             'show_plain_diffs': line_count > line_count_limit,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/search_ui.py'
--- loggerhead/controllers/search_ui.py	2008-07-26 17:41:49 +0000
+++ loggerhead/controllers/search_ui.py	2008-09-09 22:42:27 +0000
@@ -16,16 +16,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-from paste.httpexceptions import HTTPServerError
-from loggerhead import history
-from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 from loggerhead import search
 
 
 class SearchUI(TemplatedBranchView):
     """
-    
+
     Class to output progressive search result terms.
     """
 

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-08-12 16:36:23 +0000
+++ loggerhead/history.py	2008-09-09 22:47:45 +0000
@@ -199,6 +199,7 @@
             "Can only construct a History object with a read-locked branch.")
         self._file_change_cache = None
         self._branch = branch
+        self._inventory_cache = {}
         self.log = logging.getLogger('loggerhead.%s' % (branch.nick,))
 
         self.last_revid = branch.last_revision()
@@ -212,6 +213,7 @@
          self._revno_revid, self._merge_sort, self._where_merged
          ) = whole_history_data
 
+
     def use_file_cache(self, cache):
         self._file_change_cache = cache
 
@@ -417,12 +419,15 @@
             return None, None, []
 
     def get_inventory(self, revid):
-        return self._branch.repository.get_revision_inventory(revid)
+        if revid not in self._inventory_cache:
+            self._inventory_cache[revid] = (
+                self._branch.repository.get_revision_inventory(revid))
+        return self._inventory_cache[revid]
 
     def get_path(self, revid, file_id):
         if (file_id is None) or (file_id == ''):
             return ''
-        path = self._branch.repository.get_revision_inventory(revid).id2path(file_id)
+        path = self.get_inventory(revid).id2path(file_id)
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return path
@@ -430,7 +435,7 @@
     def get_file_id(self, revid, path):
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
-        return self._branch.repository.get_revision_inventory(revid).path2id(path)
+        return self.get_inventory(revid).path2id(path)
 
     def get_merge_point_list(self, revid):
         """
@@ -559,19 +564,15 @@
         trees = dict((t.get_revision_id(), t) for
                      t in self._branch.repository.revision_trees(required_trees))
         ret = []
-        self._branch.repository.lock_read()
-        try:
-            for revision in revisions:
-                if not revision.parents:
-                    old_tree = self._branch.repository.revision_tree(
-                        bzrlib.revision.NULL_REVISION)
-                else:
-                    old_tree = trees[revision.parents[0].revid]
-                tree = trees[revision.revid]
-                ret.append(tree.changes_from(old_tree))
-            return ret
-        finally:
-            self._branch.repository.unlock()
+        for revision in revisions:
+            if not revision.parents:
+                old_tree = self._branch.repository.revision_tree(
+                    bzrlib.revision.NULL_REVISION)
+            else:
+                old_tree = trees[revision.parents[0].revid]
+            tree = trees[revision.revid]
+            ret.append(tree.changes_from(old_tree))
+        return ret
 
     def _change_from_revision(self, revision):
         """

=== modified file 'loggerhead/search.py'
--- loggerhead/search.py	2008-07-26 17:41:49 +0000
+++ loggerhead/search.py	2008-09-09 22:42:27 +0000
@@ -18,7 +18,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 import sets
-import os
 try:
     from bzrlib.plugins.search import errors
     from bzrlib.plugins.search import index as _mod_index

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2008-08-13 17:33:41 +0000
+++ loggerhead/static/css/global.css	2008-09-09 22:47:45 +0000
@@ -318,4 +318,14 @@
 }
 #pages a:hover {
 	border:solid 1px #666666;
-}		
+}
+.breadcrumb a {
+	color:#889ea9;
+	font-weight:bold;
+	font-size:18px;
+}
+.breadcrumb a:hover{
+	color:#666;
+	font-weight:bold;
+	font-size:18px;
+}

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2008-08-13 19:51:08 +0000
+++ loggerhead/templatefunctions.py	2008-09-09 22:47:45 +0000
@@ -28,6 +28,7 @@
 
 
 templatefunctions['macros'] = _pt('macros').macros
+templatefunctions['breadcrumbs'] = _pt('breadcrumbs').macros
 
 @templatefunc
 def file_change_summary(url, entry, modified_file_link):

=== modified file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/annotate.pt	2008-08-16 21:53:52 +0000
@@ -22,9 +22,9 @@
         </a>
       </tal:has-link>
       <tal:no-link condition="not: branch/branch_link">
-        <tal:branch-name replace="branch/friendly_name" />
+        <span metal:use-macro="breadcrumbs/directory"></span>
       </tal:no-link>
-      <span>: <span tal:content="path"></span> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
     </h1>
 
     <div metal:fill-slot="content">

=== added file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/breadcrumbs.pt	2008-08-16 21:53:52 +0000
@@ -0,0 +1,22 @@
+<html>
+<body>
+<!--
+    Breadcrumbs macros
+-->
+<!--
+    These are all defined on one line in order to get the spacing looking right
+    in the output
+-->
+
+<!-- Branch breadcrumbs (for the path within the branch) -->
+<span metal:define-macro="branch" class="breadcrumb">
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
+</span>
+
+<!-- Directory breadcrumbs (for the path leading up to the branch) -->
+<span metal:define-macro="directory" class="breadcrumb">
+    <tal:block repeat="crumb directory_breadcrumbs"><span tal:condition="not:repeat/crumb/start">/</span><a tal:attributes="href string:/${crumb/path}${crumb/suffix}" tal:content="crumb/dir_name" /></tal:block>
+</span>
+
+</body>
+</html>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2008-08-20 20:47:46 +0000
+++ loggerhead/templates/changelog.pt	2008-09-09 22:47:45 +0000
@@ -21,7 +21,7 @@
              </a>
            </tal:has-link>
            <tal:no-link condition="not: branch/branch_link">
-             <tal:branch-name replace="branch/friendly_name" />
+             <span metal:use-macro="breadcrumbs/directory"></span>
            </tal:no-link>
            : changes
           <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2008-08-05 06:12:07 +0000
+++ loggerhead/templates/directory.pt	2008-08-16 21:53:52 +0000
@@ -7,8 +7,9 @@
   </head>
   <body>
     <div id="loggerheadCont">
-      <h1 tal:content="string:Browsing ${name}">
-        Hi.
+      <h1>
+        Browsing
+        <span metal:use-macro="breadcrumbs/directory" />
       </h1>
 
       <div >

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2008-08-06 04:20:07 +0000
+++ loggerhead/templates/inventory.pt	2008-08-16 04:57:38 +0000
@@ -17,11 +17,12 @@
           nice/branch/name
         </a>
       </tal:has-link>
-      <tal:no-link condition="not: branch/branch_link">
-        <tal:branch-name replace="branch/friendly_name" />
-      </tal:no-link>
+      <span tal:condition="not: branch/branch_link">
+        <span metal:use-macro="breadcrumbs/directory" />
+      </span>
       : viewing
-      <span tal:content="path" /> for revision <span tal:content="change/revno"/>
+      <span metal:use-macro="breadcrumbs/branch" />
+      for revision <span tal:content="change/revno"/>
     </h1>
 
     <div metal:fill-slot="content">

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2008-08-16 06:34:03 +0000
+++ loggerhead/templates/revision.pt	2008-09-09 22:47:45 +0000
@@ -23,7 +23,7 @@
             </a>
           </tal:has-link>
           <tal:no-link condition="not: branch/branch_link">
-            <tal:branch-name replace="branch/friendly_name" />
+            <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
           <span>: <tal:revno content="change/revno"></tal:revno>
         <tal:compare-to condition="python:compare_revid is not None">

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2008-07-01 04:01:31 +0000
+++ loggerhead/tests/test_simple.py	2008-09-09 22:42:27 +0000
@@ -1,5 +1,4 @@
 import cgi
-import unittest
 import os
 import tempfile
 import shutil
@@ -9,7 +8,6 @@
 import bzrlib.osutils
 from configobj import ConfigObj
 
-from loggerhead.history import History
 from loggerhead.apps.branch import BranchWSGIApp
 from paste.fixture import TestApp
 

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-09-10 00:02:06 +0000
+++ loggerhead/util.py	2008-09-10 00:07:49 +0000
@@ -32,7 +32,6 @@
 import struct
 import threading
 import time
-import types
 import sys
 import os
 import subprocess
@@ -345,6 +344,71 @@
             [navigation.scan_url, next_page_revno], **params)
 
 
+def directory_breadcrumbs(path, is_root, view):
+    """
+    Generate breadcrumb information from the directory path given
+
+    The path given should be a path up to any branch that is currently being
+    served
+
+    Arguments:
+    path -- The path to convert into breadcrumbs
+    is_root -- Whether or not loggerhead is serving a branch at its root
+    view -- The type of view we are showing (files, changes etc)
+    """
+    # Is our root directory itself a branch?
+    if is_root:
+        if view == 'directory':
+            directory = 'files'
+        breadcrumbs = [{
+            'dir_name': path,
+            'path': '',
+            'suffix': view,
+        }]
+    else:
+        # Create breadcrumb trail for the path leading up to the branch
+        breadcrumbs = [{
+            'dir_name': "(root)",
+            'path': '',
+            'suffix': '',
+        }]
+        if path != '/':
+            dir_parts = path.strip('/').split('/')
+            for index, dir_name in enumerate(dir_parts):
+                breadcrumbs.append({
+                    'dir_name': dir_name,
+                    'path': '/'.join(dir_parts[:index + 1]),
+                    'suffix': '',
+                })
+            # If we are not in the directory view, the last crumb is a branch,
+            # so we need to specify a view
+            if view != 'directory':
+                breadcrumbs[-1]['suffix'] = '/' + view
+    return breadcrumbs
+
+
+def branch_breadcrumbs(path, inv, view):
+    """
+    Generate breadcrumb information from the branch path given
+
+    The path given should be a path that exists within a branch
+
+    Arguments:
+    path -- The path to convert into breadcrumbs
+    inv -- Inventory to get file information from
+    view -- The type of view we are showing (files, changes etc)
+    """
+    dir_parts = path.strip('/').split('/')
+    inner_breadcrumbs = []
+    for index, dir_name in enumerate(dir_parts):
+        inner_breadcrumbs.append({
+            'dir_name': dir_name,
+            'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'suffix': '/' + view ,
+        })
+    return inner_breadcrumbs
+
+
 def decorator(unbound):
     def new_decorator(f):
         g = unbound(f)
@@ -510,4 +574,4 @@
             # a monitor, any exit code will restart
             if exit_code != 3:
                 return exit_code
-            print '-'*20, 'Restarting', '-'*20
\ No newline at end of file
+            print '-'*20, 'Restarting', '-'*20

